# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/core/protos/tablet.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb.core.protos import base_pb2 as ydb_dot_core_dot_protos_dot_base__pb2
from ydb.core.protos import tablet_counters_pb2 as ydb_dot_core_dot_protos_dot_tablet__counters__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cydb/core/protos/tablet.proto\x12\x11NKikimrTabletBase\x1a\x1aydb/core/protos/base.proto\x1a%ydb/core/protos/tablet_counters.proto\"\x8d\x06\n\x0cTTabletTypes\"\xfc\x05\n\x05\x45Type\x12\x0b\n\x07Unknown\x10\x00\x12\x12\n\x0eOldSchemeShard\x10\x01\x12\x10\n\x0cOldDataShard\x10\x02\x12\x0b\n\x07OldHive\x10\x03\x12\x12\n\x0eOldCoordinator\x10\x04\x12\x0c\n\x08Mediator\x10\x05\x12\x0e\n\nOldTxProxy\x10\x06\x12\x13\n\x0fOldBSController\x10\x07\x12\t\n\x05\x44ummy\x10\x08\x12\x11\n\rRTMRPartition\x10\n\x12\x0f\n\x0bOldKeyValue\x10\x0b\x12\x0c\n\x08KeyValue\x10\x0c\x12\x0f\n\x0b\x43oordinator\x10\r\x12\x08\n\x04Hive\x10\x0e\x12\x10\n\x0c\x42SController\x10\x0f\x12\x0f\n\x0bSchemeShard\x10\x10\x12\x0b\n\x07TxProxy\x10\x11\x12\r\n\tDataShard\x10\x12\x12\r\n\tPersQueue\x10\x14\x12\x07\n\x03\x43ms\x10\x15\x12\x0e\n\nNodeBroker\x10\x16\x12\x0f\n\x0bTxAllocator\x10\x17\x12\x19\n\x15PersQueueReadBalancer\x10\x18\x12\x14\n\x10\x42lockStoreVolume\x10\x19\x12\x17\n\x13\x42lockStorePartition\x10\x1a\x12\x14\n\x10TenantSlotBroker\x10\x1b\x12\x0b\n\x07\x43onsole\x10\x1c\x12\t\n\x05Kesus\x10\x1d\x12\x18\n\x14\x42lockStorePartition2\x10\x1e\x12\x1a\n\x16\x42lockStoreDiskRegistry\x10\x1f\x12\x14\n\x10SysViewProcessor\x10 \x12\r\n\tFileStore\x10!\x12\x0f\n\x0b\x43olumnShard\x10#\x12\r\n\tTestShard\x10$\x12\x11\n\rSequenceShard\x10%\x12\x19\n\x15ReplicationController\x10&\x12\r\n\tBlobDepot\x10\'\x12\x0e\n\nReserved40\x10(\x12\x0e\n\nReserved41\x10)\x12\x0e\n\nReserved42\x10*\x12\x0e\n\nReserved43\x10+\x12\x12\n\rUserTypeStart\x10\xff\x01\x12\x18\n\x0bTypeInvalid\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\"/\n\x12TTabletLogMetadata\x12\x0b\n\x03Key\x18\x01 \x01(\r\x12\x0c\n\x04\x44\x61ta\x18\x02 \x01(\x0c\"\x9f\x03\n\x0fTTabletLogEntry\x12\x11\n\tDependsOn\x18\x01 \x03(\r\x12-\n\nReferences\x18\x02 \x03(\x0b\x32\x19.NKikimrProto.TLogoBlobID\x12\x10\n\x08Snapshot\x18\x03 \x01(\x06\x12\x11\n\tConfirmed\x18\x04 \x01(\r\x12\x12\n\nIsSnapshot\x18\x05 \x01(\x08\x12/\n\x0cGcDiscovered\x18\x06 \x03(\x0b\x32\x19.NKikimrProto.TLogoBlobID\x12)\n\x06GcLeft\x18\x07 \x03(\x0b\x32\x19.NKikimrProto.TLogoBlobID\x12\x17\n\x0fIsTotalSnapshot\x18\x08 \x01(\x08\x12\x15\n\rZeroConfirmed\x18\n \x01(\x06\x12\x12\n\nZeroTailSz\x18\x0b \x01(\r\x12\x17\n\x0fZeroTailBitmask\x18\x0c \x03(\x06\x12\x17\n\x0f\x45mbeddedLogBody\x18\r \x01(\x0c\x12?\n\x10\x45mbeddedMetadata\x18\x0e \x03(\x0b\x32%.NKikimrTabletBase.TTabletLogMetadata\"\xeb\x01\n\x12TTabletChannelInfo\x12\x0f\n\x07\x43hannel\x18\x01 \x01(\r\x12\x13\n\x0b\x43hannelType\x18\x02 \x01(\r\x12\x44\n\x07History\x18\x03 \x03(\x0b\x32\x33.NKikimrTabletBase.TTabletChannelInfo.THistoryEntry\x12\x1a\n\x12\x43hannelErasureName\x18\x04 \x01(\t\x12\x13\n\x0bStoragePool\x18\x05 \x01(\t\x1a\x38\n\rTHistoryEntry\x12\x16\n\x0e\x46romGeneration\x18\x01 \x01(\r\x12\x0f\n\x07GroupID\x18\x02 \x01(\r\"\xdb\x01\n\x12TTabletStorageInfo\x12\x10\n\x08TabletID\x18\x01 \x01(\x06\x12\x37\n\x08\x43hannels\x18\x02 \x03(\x0b\x32%.NKikimrTabletBase.TTabletChannelInfo\x12\x39\n\nTabletType\x18\x03 \x01(\x0e\x32%.NKikimrTabletBase.TTabletTypes.EType\x12\x0f\n\x07Version\x18\x04 \x01(\r\x12\x15\n\rTenantIdOwner\x18\x05 \x01(\x04\x12\x17\n\x0fTenantIdLocalId\x18\x06 \x01(\x04\"*\n\x07TEvPing\x12\x10\n\x08TabletID\x18\x01 \x01(\x06\x12\r\n\x05\x46lags\x18\x02 \x01(\x04\"*\n\x07TEvPong\x12\x10\n\x08TabletID\x18\x01 \x01(\x06\x12\r\n\x05\x46lags\x18\x02 \x01(\x04\"7\n\x10TEvReadLocalBase\x12\x0f\n\x07RootKey\x18\x01 \x01(\x0c\x12\x12\n\nSaveScheme\x18\x02 \x01(\x08\"m\n\x16TEvReadLocalBaseResult\x12\x0e\n\x06Origin\x18\x01 \x01(\x04\x12\x0f\n\x07RootKey\x18\x02 \x01(\x0c\x12\x0f\n\x07IsError\x18\x03 \x01(\x08\x12\x11\n\tDocBuffer\x18\x04 \x01(\x0c\x12\x0e\n\x06Scheme\x18\x05 \x01(\x0c\">\n\x11TEvFollowerAttach\x12\x10\n\x08TabletId\x18\x01 \x01(\x06\x12\x17\n\x0f\x46ollowerAttempt\x18\x02 \x01(\r\"\xa9\x03\n\x11TEvFollowerUpdate\x12\x10\n\x08TabletId\x18\x01 \x01(\x06\x12\x17\n\x0f\x46ollowerAttempt\x18\x02 \x01(\r\x12\x15\n\rStreamCounter\x18\x03 \x01(\x04\x12\x12\n\nGeneration\x18\n \x01(\r\x12\x0c\n\x04Step\x18\x0b \x01(\r\x12\x30\n\rReferencesIds\x18\x0c \x03(\x0b\x32\x19.NKikimrProto.TLogoBlobID\x12\x12\n\nReferences\x18\r \x03(\x0c\x12\x12\n\nIsSnapshot\x18\x0e \x01(\x08\x12\x0c\n\x04\x42ody\x18\x0f \x01(\x0c\x12\x12\n\nAuxPayload\x18\x10 \x01(\x0c\x12@\n\x11TabletStorageInfo\x18\x11 \x01(\x0b\x32%.NKikimrTabletBase.TTabletStorageInfo\x12/\n\x0cGcDiscovered\x18\x14 \x03(\x0b\x32\x19.NKikimrProto.TLogoBlobID\x12)\n\x06GcLeft\x18\x15 \x03(\x0b\x32\x19.NKikimrProto.TLogoBlobID\x12\x16\n\x0eNeedGCApplyAck\x18\x16 \x01(\x08\"_\n\x10TEvFollowerGcAck\x12\x10\n\x08TabletId\x18\x01 \x01(\x06\x12\x17\n\x0f\x46ollowerAttempt\x18\x02 \x01(\r\x12\x12\n\nGeneration\x18\x03 \x01(\r\x12\x0c\n\x04Step\x18\x04 \x01(\r\"l\n\x14TEvFollowerAuxUpdate\x12\x10\n\x08TabletId\x18\x01 \x01(\x06\x12\x17\n\x0f\x46ollowerAttempt\x18\x02 \x01(\r\x12\x15\n\rStreamCounter\x18\x03 \x01(\x04\x12\x12\n\nAuxPayload\x18\n \x01(\x0c\">\n\x11TEvFollowerDetach\x12\x10\n\x08TabletId\x18\x01 \x01(\x06\x12\x17\n\x0f\x46ollowerAttempt\x18\x02 \x01(\r\"R\n\x15TEvFollowerDisconnect\x12\x10\n\x08TabletId\x18\x01 \x01(\x06\x12\x17\n\x0f\x46ollowerAttempt\x18\x02 \x01(\r\x12\x0e\n\x06Reason\x18\x03 \x01(\r\"S\n\x12TEvFollowerRefresh\x12\x10\n\x08TabletId\x18\x01 \x01(\x06\x12\x12\n\nGeneration\x18\x02 \x01(\r\x12\x17\n\x0fOfflineProtocol\x18\x03 \x01(\x08\"\x10\n\x0eTEvGetCounters\"T\n\x16TEvGetCountersResponse\x12:\n\x0eTabletCounters\x18\x01 \x01(\x0b\x32\".NKikimrTabletBase.TTabletCounters\"a\n\x13TEvCutTabletHistory\x12\x10\n\x08TabletID\x18\x01 \x01(\x06\x12\x0f\n\x07\x43hannel\x18\x02 \x01(\r\x12\x0f\n\x07GroupID\x18\x03 \x01(\r\x12\x16\n\x0e\x46romGeneration\x18\x04 \x01(\r\"I\n\x11TThroughputRecord\x12\x0f\n\x07GroupID\x18\x01 \x01(\r\x12\x0f\n\x07\x43hannel\x18\x02 \x01(\r\x12\x12\n\nThroughput\x18\x03 \x01(\x04\"=\n\x0bTIopsRecord\x12\x0f\n\x07GroupID\x18\x01 \x01(\r\x12\x0f\n\x07\x43hannel\x18\x02 \x01(\r\x12\x0c\n\x04Iops\x18\x03 \x01(\x04\"\xa6\x03\n\x08TMetrics\x12\x0b\n\x03\x43PU\x18\x01 \x01(\x04\x12\x0e\n\x06Memory\x18\x02 \x01(\x04\x12\x0f\n\x07Network\x18\x03 \x01(\x04\x12\x0f\n\x07\x43ounter\x18\x04 \x01(\x04\x12\x0f\n\x07Storage\x18\x05 \x01(\x04\x12\x41\n\x13GroupReadThroughput\x18\x06 \x03(\x0b\x32$.NKikimrTabletBase.TThroughputRecord\x12\x42\n\x14GroupWriteThroughput\x18\x07 \x03(\x0b\x32$.NKikimrTabletBase.TThroughputRecord\x12\x16\n\x0eReadThroughput\x18\x08 \x01(\x04\x12\x17\n\x0fWriteThroughput\x18\t \x01(\x04\x12\x35\n\rGroupReadIops\x18\n \x03(\x0b\x32\x1e.NKikimrTabletBase.TIopsRecord\x12\x36\n\x0eGroupWriteIops\x18\x0b \x03(\x0b\x32\x1e.NKikimrTabletBase.TIopsRecord\x12\x10\n\x08ReadIops\x18\x0c \x01(\x04\x12\x11\n\tWriteIops\x18\r \x01(\x04\"\xe5\x01\n\rTEvTabletStop\x12\x10\n\x08TabletID\x18\x01 \x01(\x04\x12\x38\n\x06Reason\x18\x02 \x01(\x0e\x32(.NKikimrTabletBase.TEvTabletStop.EReason\"\x87\x01\n\x07\x45Reason\x12\x11\n\rReasonUnknown\x10\x00\x12\x0e\n\nReasonStop\x10\x01\x12\x11\n\rReasonDemoted\x10\x02\x12\x12\n\x0eReasonIsolated\x10\x03\x12\x18\n\x14ReasonStorageBlocked\x10\x04\x12\x18\n\x14ReasonStorageFailure\x10\x05\" \n\x0cTEvDropLease\x12\x10\n\x08TabletID\x18\x01 \x01(\x04\"#\n\x0fTEvLeaseDropped\x12\x10\n\x08TabletID\x18\x01 \x01(\x04\x42\x18\n\x16ru.yandex.kikimr.proto')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.core.protos.tablet_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ru.yandex.kikimr.proto'
  _TTABLETTYPES._serialized_start=119
  _TTABLETTYPES._serialized_end=900
  _TTABLETTYPES_ETYPE._serialized_start=136
  _TTABLETTYPES_ETYPE._serialized_end=900
  _TTABLETLOGMETADATA._serialized_start=902
  _TTABLETLOGMETADATA._serialized_end=949
  _TTABLETLOGENTRY._serialized_start=952
  _TTABLETLOGENTRY._serialized_end=1367
  _TTABLETCHANNELINFO._serialized_start=1370
  _TTABLETCHANNELINFO._serialized_end=1605
  _TTABLETCHANNELINFO_THISTORYENTRY._serialized_start=1549
  _TTABLETCHANNELINFO_THISTORYENTRY._serialized_end=1605
  _TTABLETSTORAGEINFO._serialized_start=1608
  _TTABLETSTORAGEINFO._serialized_end=1827
  _TEVPING._serialized_start=1829
  _TEVPING._serialized_end=1871
  _TEVPONG._serialized_start=1873
  _TEVPONG._serialized_end=1915
  _TEVREADLOCALBASE._serialized_start=1917
  _TEVREADLOCALBASE._serialized_end=1972
  _TEVREADLOCALBASERESULT._serialized_start=1974
  _TEVREADLOCALBASERESULT._serialized_end=2083
  _TEVFOLLOWERATTACH._serialized_start=2085
  _TEVFOLLOWERATTACH._serialized_end=2147
  _TEVFOLLOWERUPDATE._serialized_start=2150
  _TEVFOLLOWERUPDATE._serialized_end=2575
  _TEVFOLLOWERGCACK._serialized_start=2577
  _TEVFOLLOWERGCACK._serialized_end=2672
  _TEVFOLLOWERAUXUPDATE._serialized_start=2674
  _TEVFOLLOWERAUXUPDATE._serialized_end=2782
  _TEVFOLLOWERDETACH._serialized_start=2784
  _TEVFOLLOWERDETACH._serialized_end=2846
  _TEVFOLLOWERDISCONNECT._serialized_start=2848
  _TEVFOLLOWERDISCONNECT._serialized_end=2930
  _TEVFOLLOWERREFRESH._serialized_start=2932
  _TEVFOLLOWERREFRESH._serialized_end=3015
  _TEVGETCOUNTERS._serialized_start=3017
  _TEVGETCOUNTERS._serialized_end=3033
  _TEVGETCOUNTERSRESPONSE._serialized_start=3035
  _TEVGETCOUNTERSRESPONSE._serialized_end=3119
  _TEVCUTTABLETHISTORY._serialized_start=3121
  _TEVCUTTABLETHISTORY._serialized_end=3218
  _TTHROUGHPUTRECORD._serialized_start=3220
  _TTHROUGHPUTRECORD._serialized_end=3293
  _TIOPSRECORD._serialized_start=3295
  _TIOPSRECORD._serialized_end=3356
  _TMETRICS._serialized_start=3359
  _TMETRICS._serialized_end=3781
  _TEVTABLETSTOP._serialized_start=3784
  _TEVTABLETSTOP._serialized_end=4013
  _TEVTABLETSTOP_EREASON._serialized_start=3878
  _TEVTABLETSTOP_EREASON._serialized_end=4013
  _TEVDROPLEASE._serialized_start=4015
  _TEVDROPLEASE._serialized_end=4047
  _TEVLEASEDROPPED._serialized_start=4049
  _TEVLEASEDROPPED._serialized_end=4084
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/library/yql/protos/common.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#ydb/library/yql/protos/common.proto\x12\x0bNYql.NProto\"\xab\x01\n\x05TTime\x12\r\n\x05Value\x18\x01 \x01(\x04\x12&\n\x04Unit\x18\x02 \x01(\x0e\x32\x18.NYql.NProto.TTime.TUnit\"k\n\x05TUnit\x12\x0f\n\x0bNANOSECONDS\x10\x01\x12\x10\n\x0cMICROSECONDS\x10\x02\x12\x10\n\x0cMILLISECONDS\x10\x03\x12\x0b\n\x07SECONDS\x10\x04\x12\x0b\n\x07MINUTES\x10\x05\x12\t\n\x05HOURS\x10\x06\x12\x08\n\x04\x44\x41YS\x10\x07\x42\x15\n\x13ru.yandex.yql.proto')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.library.yql.protos.common_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023ru.yandex.yql.proto'
  _TTIME._serialized_start=53
  _TTIME._serialized_end=224
  _TTIME_TUNIT._serialized_start=117
  _TTIME_TUNIT._serialized_end=224
# @@protoc_insertion_point(module_scope)



import ydb.core.protos.blobstorage_config_pb2 as kikimr_bsconfig
import ydb.apps.dstool.lib.common as common
import sys

description = 'Set pdisk properties. It may impact respective vdisks'


def add_options(p):
    common.add_pdisk_ids_option(p, required=True)
    g = p.add_mutually_exclusive_group(required=True)
    statuses = kikimr_bsconfig.EDriveStatus.keys()
    g.add_argument('--status', type=str, choices=statuses, help='Set status')
    decommit_statuses = kikimr_bsconfig.EDecommitStatus.keys()
    g.add_argument('--decommit-status', type=str, choices=decommit_statuses, help='Set decomission status')
    common.add_allow_unusable_pdisks_option(p)
    p.add_argument('--allow-working-disks', action='store_true', help='Allow settlement even if any of enlisted PDisks is still working')
    common.add_ignore_degraded_group_check_option(p)
    common.add_ignore_failure_model_group_check_option(p)
    common.add_ignore_vslot_quotas_option(p)
    p.add_argument('--unavail-as-offline', action='store_true', help='Treat PDisks not reported by Node Whiteboard as offline')
    common.add_basic_format_options(p)


def create_request(args, pdisks, node_id_to_host):
    request = common.create_bsc_request(args)
    for pdisk_id, pdisk in pdisks.items():
        cmd = request.Command.add().UpdateDriveStatus
        cmd.HostKey.Fqdn, cmd.HostKey.IcPort = node_id_to_host[pdisk.NodeId]
        if pdisk.Path:
            cmd.Path = pdisk.Path
        else:
            _, cmd.PDiskId = pdisk_id
        if args.status is not None:
            cmd.Status = kikimr_bsconfig.EDriveStatus.Value(args.status)
        if args.decommit_status is not None:
            cmd.DecommitStatus = kikimr_bsconfig.EDecommitStatus.Value(args.decommit_status)

    return request


def perform_request(request):
    return common.invoke_bsc_request(request)


def is_successful_response(response):
    return common.is_successful_bsc_response(response)


def get_pdisks(pdisk_ids, base_config):
    pdisks = {}
    for arg in pdisk_ids:
        for id in arg.split():
            # id is in the form '[NodeId:PDiskId]'
            node, pdisk = id.strip('[').strip(']').split(':')
            pdisk_id = int(node), int(pdisk)
            pdisks[pdisk_id] = None

    for pdisk in base_config.PDisk:
        pdisk_id = common.get_pdisk_id(pdisk)
        if pdisk_id in pdisks:
            pdisks[pdisk_id] = pdisk

    return pdisks


def do(args):
    base_config = common.fetch_base_config()
    pdisks = get_pdisks(args.pdisk_ids, base_config)
    node_id_to_host, _ = common.build_node_fqdn_maps(base_config)

    for pdisk_id, pdisk in pdisks.items():
        if pdisk is None:
            common.print_status(args, success=False, error_reason='Unknown pdisk id [%d:%d]' % pdisk_id)
            sys.exit(1)
        node_id = pdisk_id[0]
        if node_id not in node_id_to_host:
            common.print_status(args, success=False, error_reason="Can't determine FQDN for node id %d" % node_id)
            sys.exit(1)

    if not args.allow_working_disks and args.status == 'BROKEN':
        params = dict(node_id=0, enums=1)
        error_reason = ''
        success = True
        pdisk_status = {}
        pdisk_name = {
            pdisk_id: 'PDisk %d:%d (%s:%s)' % (pdisk_id + (node_id_to_host.get(pdisk.NodeId, [None, None])[0], pdisk.Path))
            for pdisk_id, pdisk in pdisks.items()
        }
        for pdisk_id, pdisk in sorted(pdisks.items()):
            try:
                for row in common.fetch('viewer/json/pdiskinfo', params, node_id_to_host[pdisk.NodeId][0]).get('PDiskStateInfo', []):
                    if 'State' in row and row['NodeId'] == pdisk.NodeId and row['PDiskId'] == pdisk.PDiskId:
                        pdisk_status[pdisk_id] = not row['State'].endswith('Error')
            except Exception as e:
                if args.unavail_as_offline:
                    pdisk_status[pdisk_id] = False
                else:
                    pdisk_status[pdisk_id] = None
                    error_reason += "Can't determine %s status: %s\n" % (pdisk_name[pdisk_id], e)
                    success = False
        for pdisk_id, pdisk in sorted(pdisks.items()):
            if pdisk_id not in pdisk_status:
                error_reason += '%s is not found in node JSON monitoring\n' % pdisk_name[pdisk_id]
                success = False
            elif pdisk_status[pdisk_id]:
                error_reason += '%s is reported as working one\n' % pdisk_name[pdisk_id]
                success = False
        if not success:
            common.print_status(args, success, error_reason=error_reason)
            sys.exit(1)

    request = create_request(args, pdisks, node_id_to_host)
    response = perform_request(request)
    common.print_request_result(args, request, response)
    if not is_successful_response(response):
        sys.exit(1)



import ydb.apps.dstool.lib.common as common
import sys

description = 'Change or query group state'


def add_options(p):
    common.add_group_ids_option(p, required=True)
    g = p.add_mutually_exclusive_group(required=True)
    g.add_argument('--query', action='store_true', help='Query group state')
    g.add_argument('--down', action='store_true', help='Change group state to down')
    g.add_argument('--up', action='store_true', help='Change group state back to up')
    p.add_argument('--persist', action='store_true', help='Save changes to blog storage controller')


def do(args):
    controller_id = 0x000000000001001 | common.connection_params.domain << 56

    if args.query:
        res = []
        for group_id in args.group_ids:
            res.append(common.fetch('tablets/app', dict(TabletID=controller_id, page='GetDown', group=group_id)))
        for item in res:
            print('GroupId# {GroupId} Down# {Down} PersistedDown# {PersistedDown}'.format(**item))
    else:
        error_reason = ''
        success = True
        for group_id in args.group_ids:
            res = common.fetch('tablets/app', dict(TabletID=controller_id, page='SetDown', group=group_id, down=int(args.down), persist=int(args.persist)))
            if 'Error' in res:
                error_reason += 'GroupId# {GroupId} Error# {Error}\n'.format(GroupId=group_id, **res)
                success = False
        common.print_status(args, success, error_reason)
        if not success:
            sys.exit(1)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/library/yql/dq/actors/protos/dq_stats.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/ydb/library/yql/dq/actors/protos/dq_stats.proto\x12\rNYql.NDqProto\x1a\x19google/protobuf/any.proto\"\xd6\x01\n\x18TDqAsyncInputBufferStats\x12\x12\n\nInputIndex\x18\x01 \x01(\x04\x12\x0e\n\x06\x43hunks\x18\x02 \x01(\x04\x12\r\n\x05\x42ytes\x18\x03 \x01(\x04\x12\x0e\n\x06RowsIn\x18\x04 \x01(\x04\x12\x0f\n\x07RowsOut\x18\x05 \x01(\x04\x12\x16\n\x0eMaxMemoryUsage\x18\x06 \x01(\x04\x12\x13\n\x0b\x45rrorsCount\x18\x07 \x01(\r\x12\x14\n\x0cIngressBytes\x18\x08 \x01(\x04\x12#\n\x05\x45xtra\x18\x64 \x01(\x0b\x32\x14.google.protobuf.Any\"\x87\x02\n\x14TDqInputChannelStats\x12\x11\n\tChannelId\x18\x01 \x01(\x04\x12\x0e\n\x06\x43hunks\x18\x02 \x01(\x04\x12\r\n\x05\x42ytes\x18\x03 \x01(\x04\x12\x0e\n\x06RowsIn\x18\x04 \x01(\x04\x12\x0f\n\x07RowsOut\x18\x05 \x01(\x04\x12\x1d\n\x15\x44\x65serializationTimeUs\x18\x06 \x01(\x04\x12\x16\n\x0eMaxMemoryUsage\x18\x07 \x01(\x04\x12\x14\n\x0cPollRequests\x18\x08 \x01(\r\x12\x16\n\x0eResentMessages\x18\t \x01(\r\x12\x12\n\nWaitTimeUs\x18\n \x01(\x04\x12#\n\x05\x45xtra\x18\x64 \x01(\x0b\x32\x14.google.protobuf.Any\"\xd7\x01\n\x19TDqAsyncOutputBufferStats\x12\x13\n\x0bOutputIndex\x18\x01 \x01(\x04\x12\x0e\n\x06\x43hunks\x18\x02 \x01(\x04\x12\r\n\x05\x42ytes\x18\x03 \x01(\x04\x12\x0e\n\x06RowsIn\x18\x04 \x01(\x04\x12\x0f\n\x07RowsOut\x18\x05 \x01(\x04\x12\x16\n\x0eMaxMemoryUsage\x18\x06 \x01(\x04\x12\x13\n\x0b\x45rrorsCount\x18\x07 \x01(\r\x12\x13\n\x0b\x45gressBytes\x18\x08 \x01(\x04\x12#\n\x05\x45xtra\x18\x64 \x01(\x0b\x32\x14.google.protobuf.Any\"\xe7\x02\n\x15TDqOutputChannelStats\x12\x11\n\tChannelId\x18\x01 \x01(\x04\x12\x0e\n\x06\x43hunks\x18\x02 \x01(\x04\x12\r\n\x05\x42ytes\x18\x03 \x01(\x04\x12\x0e\n\x06RowsIn\x18\x04 \x01(\x04\x12\x0f\n\x07RowsOut\x18\x05 \x01(\x04\x12\x1b\n\x13SerializationTimeUs\x18\x06 \x01(\x04\x12\x19\n\x11\x42lockedByCapacity\x18\x07 \x01(\r\x12\x16\n\x0eResentMessages\x18\x08 \x01(\r\x12\x14\n\x0cNoDstActorId\x18\t \x01(\r\x12\x17\n\x0fMaxRowsInMemory\x18\n \x01(\r\x12\x16\n\x0eMaxMemoryUsage\x18\x0b \x01(\x04\x12\x14\n\x0cSpilledBytes\x18\x0c \x01(\x04\x12\x13\n\x0bSpilledRows\x18\r \x01(\x04\x12\x14\n\x0cSpilledBlobs\x18\x0e \x01(\x04\x12#\n\x05\x45xtra\x18\x64 \x01(\x0b\x32\x14.google.protobuf.Any\"\xd6\x01\n\rTDqTableStats\x12\x11\n\tTablePath\x18\x01 \x01(\t\x12\x10\n\x08ReadRows\x18\x02 \x01(\x04\x12\x11\n\tReadBytes\x18\x03 \x01(\x04\x12\x11\n\tWriteRows\x18\x04 \x01(\x04\x12\x12\n\nWriteBytes\x18\x05 \x01(\x04\x12\x11\n\tEraseRows\x18\x06 \x01(\x04\x12\x12\n\nEraseBytes\x18\x07 \x01(\x04\x12\x1a\n\x12\x41\x66\x66\x65\x63tedPartitions\x18\x08 \x01(\r\x12#\n\x05\x45xtra\x18\x64 \x01(\x0b\x32\x14.google.protobuf.Any\"9\n\x0bTDqMkqlStat\x12\x0c\n\x04Name\x18\x01 \x01(\t\x12\r\n\x05Value\x18\x02 \x01(\x03\x12\r\n\x05\x44\x65riv\x18\x03 \x01(\x08\"3\n\x14TDqDataProviderStats\x12\x0c\n\x04Name\x18\x01 \x01(\t\x12\r\n\x05\x42ytes\x18\x02 \x01(\x04\"\x8c\x08\n\x0cTDqTaskStats\x12\x0e\n\x06TaskId\x18\x01 \x01(\x04\x12\x0f\n\x07StageId\x18\x02 \x01(\r\x12\x11\n\tCpuTimeUs\x18\x03 \x01(\x04\x12\x16\n\x0e\x46irstRowTimeMs\x18\x04 \x01(\x04\x12\x14\n\x0c\x46inishTimeMs\x18\x05 \x01(\x04\x12\x11\n\tInputRows\x18\x06 \x01(\x04\x12\x12\n\nInputBytes\x18\x07 \x01(\x04\x12\x12\n\nOutputRows\x18\x08 \x01(\x04\x12\x13\n\x0bOutputBytes\x18\t \x01(\x04\x12,\n\x06Tables\x18\n \x03(\x0b\x32\x1c.NYql.NDqProto.TDqTableStats\x12\x34\n\x07Ingress\x18\r \x03(\x0b\x32#.NYql.NDqProto.TDqDataProviderStats\x12\x33\n\x06\x45gress\x18\x0e \x03(\x0b\x32#.NYql.NDqProto.TDqDataProviderStats\x12\x16\n\x0e\x42uildCpuTimeUs\x18g \x01(\x04\x12\x12\n\nWaitTimeUs\x18h \x01(\x04\x12\x18\n\x10WaitOutputTimeUs\x18i \x01(\x04\x12\x18\n\x10\x43omputeCpuTimeUs\x18\x66 \x01(\x04\x12\x1a\n\x12PendingInputTimeUs\x18k \x01(\x04\x12\x1b\n\x13PendingOutputTimeUs\x18l \x01(\x04\x12\x14\n\x0c\x46inishTimeUs\x18m \x01(\x04\x12-\n\tMkqlStats\x18n \x03(\x0b\x32\x1a.NYql.NDqProto.TDqMkqlStat\x12\x44\n\x13\x43omputeCpuTimeByRun\x18j \x03(\x0b\x32\'.NYql.NDqProto.TDqTaskStats.THistBucket\x12\x39\n\x07Sources\x18\x96\x01 \x03(\x0b\x32\'.NYql.NDqProto.TDqAsyncInputBufferStats\x12;\n\rInputChannels\x18\x97\x01 \x03(\x0b\x32#.NYql.NDqProto.TDqInputChannelStats\x12\x38\n\x05Sinks\x18\x98\x01 \x03(\x0b\x32(.NYql.NDqProto.TDqAsyncOutputBufferStats\x12=\n\x0eOutputChannels\x18\x99\x01 \x03(\x0b\x32$.NYql.NDqProto.TDqOutputChannelStats\x12\x41\n\x0fInputTransforms\x18\x9b\x01 \x03(\x0b\x32\'.NYql.NDqProto.TDqAsyncInputBufferStats\x12$\n\x05\x45xtra\x18\xc8\x01 \x01(\x0b\x32\x14.google.protobuf.Any\x1a+\n\x0bTHistBucket\x12\r\n\x05\x42ound\x18\x01 \x01(\x01\x12\r\n\x05Value\x18\x02 \x01(\x04J\x06\x08\x9a\x01\x10\x9b\x01\"\xea\x01\n\x14TDqComputeActorStats\x12\x11\n\tCpuTimeUs\x18\x01 \x01(\x04\x12\x12\n\nDurationUs\x18\x02 \x01(\x04\x12*\n\x05Tasks\x18\x03 \x03(\x0b\x32\x1b.NYql.NDqProto.TDqTaskStats\x12\x1a\n\x12MkqlMaxMemoryUsage\x18\x66 \x01(\x04\x12\x1c\n\x14MkqlExtraMemoryBytes\x18g \x01(\x04\x12\x1f\n\x17MkqlExtraMemoryRequests\x18h \x01(\r\x12$\n\x05\x45xtra\x18\xc8\x01 \x01(\x0b\x32\x14.google.protobuf.Any\"B\n\x0cTDqStatsAggr\x12\x0b\n\x03Min\x18\x01 \x01(\x04\x12\x0b\n\x03Max\x18\x02 \x01(\x04\x12\x0b\n\x03Sum\x18\x03 \x01(\x04\x12\x0b\n\x03\x43nt\x18\x04 \x01(\x04\"\x8e\x01\n\x0bTExtraStats\x12\x34\n\x05Stats\x18\x01 \x03(\x0b\x32%.NYql.NDqProto.TExtraStats.StatsEntry\x1aI\n\nStatsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12*\n\x05value\x18\x02 \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr:\x02\x38\x01\"*\n\x0eTDqStatsMinMax\x12\x0b\n\x03Min\x18\x01 \x01(\x04\x12\x0b\n\x03Max\x18\x02 \x01(\x04\"\x88\x03\n\x11TDqTableAggrStats\x12\x11\n\tTablePath\x18\x01 \x01(\t\x12-\n\x08ReadRows\x18\x02 \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12.\n\tReadBytes\x18\x03 \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12.\n\tWriteRows\x18\x04 \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12/\n\nWriteBytes\x18\x05 \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12.\n\tEraseRows\x18\x06 \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12/\n\nEraseBytes\x18\x07 \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12\x1a\n\x12\x41\x66\x66\x65\x63tedPartitions\x18\x08 \x01(\r\x12#\n\x05\x45xtra\x18\x64 \x01(\x0b\x32\x14.google.protobuf.Any\"\xfe\x04\n\rTDqStageStats\x12\x0f\n\x07StageId\x18\x01 \x01(\r\x12\x11\n\tStageGuid\x18\x02 \x01(\t\x12\x0f\n\x07Program\x18\x03 \x01(\t\x12\x17\n\x0fTotalTasksCount\x18\x05 \x01(\r\x12\x18\n\x10\x46\x61iledTasksCount\x18\x06 \x01(\r\x12.\n\tCpuTimeUs\x18\x08 \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12.\n\tInputRows\x18\t \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12/\n\nInputBytes\x18\n \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12/\n\nOutputRows\x18\x0b \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12\x30\n\x0bOutputBytes\x18\x0c \x01(\x0b\x32\x1b.NYql.NDqProto.TDqStatsAggr\x12\x35\n\x0e\x46irstRowTimeMs\x18\r \x01(\x0b\x32\x1d.NYql.NDqProto.TDqStatsMinMax\x12\x33\n\x0c\x46inishTimeMs\x18\x0e \x01(\x0b\x32\x1d.NYql.NDqProto.TDqStatsMinMax\x12\x12\n\nDurationUs\x18\x0f \x01(\x04\x12\x30\n\x06Tables\x18\x10 \x03(\x0b\x32 .NYql.NDqProto.TDqTableAggrStats\x12:\n\rComputeActors\x18\x11 \x03(\x0b\x32#.NYql.NDqProto.TDqComputeActorStats\x12#\n\x05\x45xtra\x18\x64 \x01(\x0b\x32\x14.google.protobuf.Any\"\xdc\x02\n\x11TDqExecutionStats\x12\x11\n\tCpuTimeUs\x18\x01 \x01(\x04\x12\x12\n\nDurationUs\x18\x02 \x01(\x04\x12\x12\n\nResultRows\x18\x03 \x01(\x04\x12\x13\n\x0bResultBytes\x18\x04 \x01(\x04\x12,\n\x06Tables\x18\x06 \x03(\x0b\x32\x1c.NYql.NDqProto.TDqTableStats\x12\x19\n\x11\x45xecuterCpuTimeUs\x18\n \x01(\x04\x12\x13\n\x0bStartTimeMs\x18\x0b \x01(\x04\x12\x14\n\x0c\x46inishTimeMs\x18\x0c \x01(\x04\x12\x16\n\x0e\x46irstRowTimeMs\x18\r \x01(\x04\x12,\n\x06Stages\x18\x0e \x03(\x0b\x32\x1c.NYql.NDqProto.TDqStageStats\x12\x18\n\x10TxPlansWithStats\x18\x0f \x03(\t\x12#\n\x05\x45xtra\x18\x64 \x01(\x0b\x32\x14.google.protobuf.Any*y\n\x0c\x45\x44qStatsMode\x12\x1d\n\x19\x44Q_STATS_MODE_UNSPECIFIED\x10\x00\x12\x16\n\x12\x44Q_STATS_MODE_NONE\x10\n\x12\x17\n\x13\x44Q_STATS_MODE_BASIC\x10\x14\x12\x19\n\x15\x44Q_STATS_MODE_PROFILE\x10\x1e\x42\x03\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.library.yql.dq.actors.protos.dq_stats_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\370\001\001'
  _TEXTRASTATS_STATSENTRY._options = None
  _TEXTRASTATS_STATSENTRY._serialized_options = b'8\001'
  _EDQSTATSMODE._serialized_start=4405
  _EDQSTATSMODE._serialized_end=4526
  _TDQASYNCINPUTBUFFERSTATS._serialized_start=94
  _TDQASYNCINPUTBUFFERSTATS._serialized_end=308
  _TDQINPUTCHANNELSTATS._serialized_start=311
  _TDQINPUTCHANNELSTATS._serialized_end=574
  _TDQASYNCOUTPUTBUFFERSTATS._serialized_start=577
  _TDQASYNCOUTPUTBUFFERSTATS._serialized_end=792
  _TDQOUTPUTCHANNELSTATS._serialized_start=795
  _TDQOUTPUTCHANNELSTATS._serialized_end=1154
  _TDQTABLESTATS._serialized_start=1157
  _TDQTABLESTATS._serialized_end=1371
  _TDQMKQLSTAT._serialized_start=1373
  _TDQMKQLSTAT._serialized_end=1430
  _TDQDATAPROVIDERSTATS._serialized_start=1432
  _TDQDATAPROVIDERSTATS._serialized_end=1483
  _TDQTASKSTATS._serialized_start=1486
  _TDQTASKSTATS._serialized_end=2522
  _TDQTASKSTATS_THISTBUCKET._serialized_start=2471
  _TDQTASKSTATS_THISTBUCKET._serialized_end=2514
  _TDQCOMPUTEACTORSTATS._serialized_start=2525
  _TDQCOMPUTEACTORSTATS._serialized_end=2759
  _TDQSTATSAGGR._serialized_start=2761
  _TDQSTATSAGGR._serialized_end=2827
  _TEXTRASTATS._serialized_start=2830
  _TEXTRASTATS._serialized_end=2972
  _TEXTRASTATS_STATSENTRY._serialized_start=2899
  _TEXTRASTATS_STATSENTRY._serialized_end=2972
  _TDQSTATSMINMAX._serialized_start=2974
  _TDQSTATSMINMAX._serialized_end=3016
  _TDQTABLEAGGRSTATS._serialized_start=3019
  _TDQTABLEAGGRSTATS._serialized_end=3411
  _TDQSTAGESTATS._serialized_start=3414
  _TDQSTAGESTATS._serialized_end=4052
  _TDQEXECUTIONSTATS._serialized_start=4055
  _TDQEXECUTIONSTATS._serialized_end=4403
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/core/protos/counters_cms.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb.core.protos import counters_pb2 as ydb_dot_core_dot_protos_dot_counters__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"ydb/core/protos/counters_cms.proto\x12\x0cNKikimr.NCms\x1a\x1eydb/core/protos/counters.proto*t\n\x0f\x45SimpleCounters\x12\x19\n\x15\x43OUNTER_SIMPLE_IGNORE\x10\x00\x12\x46\n\x19\x43OUNTER_BOOTSTRAP_DIFFERS\x10\x01\x1a\'\x82\xd6\x1b#\n!BootstrapConfigDiffersFromConsole*4\n\x13\x45\x43umulativeCounters\x12\x1d\n\x19\x43OUNTER_CUMULATIVE_IGNORE\x10\x00*N\n\x13\x45PercentileCounters\x12\x1d\n\x19\x43OUNTER_PERCENTILE_IGNORE\x10\x00\x1a\x18\x92\xd6\x1b\x14\x12\x08\x08\x00\x12\x04\x30 ms\x12\x08\x08\x01\x12\x04\x31 ms*\xc0\x06\n\x08\x45TxTypes\x12*\n\x12TXTYPE_INIT_SCHEMA\x10\x00\x1a\x12\x8a\xd6\x1b\x0e\n\x0cTxInitSchema\x12\x1d\n\x0bTXTYPE_INIT\x10\x01\x1a\x0c\x8a\xd6\x1b\x08\n\x06TxInit\x12+\n\x13TXTYPE_GET_LOG_TAIL\x10\x02\x1a\x12\x8a\xd6\x1b\x0e\n\x0cTxGetLogTail\x12(\n\x11TXTYPE_LOAD_STATE\x10\x03\x1a\x11\x8a\xd6\x1b\r\n\x0bTxLoadState\x12+\n\x13TXTYPE_LOG_AND_SEND\x10\x04\x1a\x12\x8a\xd6\x1b\x0e\n\x0cTxLogAndSend\x12*\n\x12TXTYPE_LOG_CLEANUP\x10\x05\x1a\x12\x8a\xd6\x1b\x0e\n\x0cTxLogCleanup\x12<\n\x1bTXTYPE_PROCESS_NOTIFICATION\x10\x06\x1a\x1b\x8a\xd6\x1b\x17\n\x15TxProcessNotification\x12:\n\x1aTXTYPE_REJECT_NOTIFICATION\x10\x07\x1a\x1a\x8a\xd6\x1b\x16\n\x14TxRejectNotification\x12I\n\"TXTYPE_REMOVE_EXPIRED_NOTIFICATION\x10\x08\x1a!\x8a\xd6\x1b\x1d\n\x1bTxRemoveExpiredNotification\x12\x36\n\x18TXTYPE_STORE_PERMISSIONS\x10\t\x1a\x18\x8a\xd6\x1b\x14\n\x12TxStorePermissions\x12\x38\n\x19TXTYPE_REMOVE_PERMISSIONS\x10\n\x1a\x19\x8a\xd6\x1b\x15\n\x13TxRemovePermissions\x12\x30\n\x15TXTYPE_REMOVE_REQUEST\x10\x0b\x1a\x15\x8a\xd6\x1b\x11\n\x0fTxRemoveRequest\x12\x35\n\x18TXTYPE_REMOVE_WALLE_TASK\x10\x0c\x1a\x17\x8a\xd6\x1b\x13\n\x11TxRemoveWalleTask\x12\x33\n\x17TXTYPE_STORE_WALLE_TASK\x10\r\x1a\x16\x8a\xd6\x1b\x12\n\x10TxStoreWalleTask\x12.\n\x14TXTYPE_UPDATE_CONFIG\x10\x0e\x1a\x14\x8a\xd6\x1b\x10\n\x0eTxUpdateConfig\x12\x34\n\x17TXTYPE_UPDATE_DOWNTIMES\x10\x0f\x1a\x17\x8a\xd6\x1b\x13\n\x11TxUpdateDowntimesB\x1f\n\x16ru.yandex.kikimr.proto\x9a\xd6\x1b\x03\x43ms')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.core.protos.counters_cms_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ru.yandex.kikimr.proto\232\326\033\003Cms'
  _ESIMPLECOUNTERS.values_by_name["COUNTER_BOOTSTRAP_DIFFERS"]._options = None
  _ESIMPLECOUNTERS.values_by_name["COUNTER_BOOTSTRAP_DIFFERS"]._serialized_options = b'\202\326\033#\n!BootstrapConfigDiffersFromConsole'
  _EPERCENTILECOUNTERS._options = None
  _EPERCENTILECOUNTERS._serialized_options = b'\222\326\033\024\022\010\010\000\022\0040 ms\022\010\010\001\022\0041 ms'
  _ETXTYPES.values_by_name["TXTYPE_INIT_SCHEMA"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_INIT_SCHEMA"]._serialized_options = b'\212\326\033\016\n\014TxInitSchema'
  _ETXTYPES.values_by_name["TXTYPE_INIT"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_INIT"]._serialized_options = b'\212\326\033\010\n\006TxInit'
  _ETXTYPES.values_by_name["TXTYPE_GET_LOG_TAIL"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_GET_LOG_TAIL"]._serialized_options = b'\212\326\033\016\n\014TxGetLogTail'
  _ETXTYPES.values_by_name["TXTYPE_LOAD_STATE"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_LOAD_STATE"]._serialized_options = b'\212\326\033\r\n\013TxLoadState'
  _ETXTYPES.values_by_name["TXTYPE_LOG_AND_SEND"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_LOG_AND_SEND"]._serialized_options = b'\212\326\033\016\n\014TxLogAndSend'
  _ETXTYPES.values_by_name["TXTYPE_LOG_CLEANUP"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_LOG_CLEANUP"]._serialized_options = b'\212\326\033\016\n\014TxLogCleanup'
  _ETXTYPES.values_by_name["TXTYPE_PROCESS_NOTIFICATION"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_PROCESS_NOTIFICATION"]._serialized_options = b'\212\326\033\027\n\025TxProcessNotification'
  _ETXTYPES.values_by_name["TXTYPE_REJECT_NOTIFICATION"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_REJECT_NOTIFICATION"]._serialized_options = b'\212\326\033\026\n\024TxRejectNotification'
  _ETXTYPES.values_by_name["TXTYPE_REMOVE_EXPIRED_NOTIFICATION"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_REMOVE_EXPIRED_NOTIFICATION"]._serialized_options = b'\212\326\033\035\n\033TxRemoveExpiredNotification'
  _ETXTYPES.values_by_name["TXTYPE_STORE_PERMISSIONS"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_STORE_PERMISSIONS"]._serialized_options = b'\212\326\033\024\n\022TxStorePermissions'
  _ETXTYPES.values_by_name["TXTYPE_REMOVE_PERMISSIONS"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_REMOVE_PERMISSIONS"]._serialized_options = b'\212\326\033\025\n\023TxRemovePermissions'
  _ETXTYPES.values_by_name["TXTYPE_REMOVE_REQUEST"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_REMOVE_REQUEST"]._serialized_options = b'\212\326\033\021\n\017TxRemoveRequest'
  _ETXTYPES.values_by_name["TXTYPE_REMOVE_WALLE_TASK"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_REMOVE_WALLE_TASK"]._serialized_options = b'\212\326\033\023\n\021TxRemoveWalleTask'
  _ETXTYPES.values_by_name["TXTYPE_STORE_WALLE_TASK"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_STORE_WALLE_TASK"]._serialized_options = b'\212\326\033\022\n\020TxStoreWalleTask'
  _ETXTYPES.values_by_name["TXTYPE_UPDATE_CONFIG"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_UPDATE_CONFIG"]._serialized_options = b'\212\326\033\020\n\016TxUpdateConfig'
  _ETXTYPES.values_by_name["TXTYPE_UPDATE_DOWNTIMES"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_UPDATE_DOWNTIMES"]._serialized_options = b'\212\326\033\023\n\021TxUpdateDowntimes'
  _ESIMPLECOUNTERS._serialized_start=84
  _ESIMPLECOUNTERS._serialized_end=200
  _ECUMULATIVECOUNTERS._serialized_start=202
  _ECUMULATIVECOUNTERS._serialized_end=254
  _EPERCENTILECOUNTERS._serialized_start=256
  _EPERCENTILECOUNTERS._serialized_end=334
  _ETXTYPES._serialized_start=337
  _ETXTYPES._serialized_end=1169
# @@protoc_insertion_point(module_scope)



import ydb.core.protos.blobstorage_config_pb2 as kikimr_bsconfig
import ydb.apps.dstool.lib.common as common
import ydb.apps.dstool.lib.table as table
import math
from collections import defaultdict

description = 'List pools'


def add_options(p):
    p.add_argument('--show-group-status', action='store_true', help='Show columns with Group status')
    p.add_argument('--show-vdisk-status', action='store_true', help='Show columns with VDisk status')
    p.add_argument('--show-vdisk-usage', action='store_true', help='Show columns with VDisk usage')
    p.add_argument('--show-vdisk-estimated-usage', action='store_true', help='Show columns with VDisk estimated usage')
    table.TableOutput([], col_units=[]).add_options(p)


def apply_func(func, arg):
    if len(arg) > 0:
        return func(arg)
    else:
        return 0


def calculate_estimated_usage(pdisk_map, pdisk_slot_usage_map, vslot_map, groups):
    max_used_size = 0
    min_fair_size = 0
    vslot_fair_usages = []
    for group in groups:
        vslot_fair_sizes = []
        vslot_used_sizes = []

        for vslot in common.vslots_of_group(group, vslot_map):
            pdisk_id = common.get_pdisk_id(vslot.VSlotId)
            if pdisk_id not in pdisk_map:
                continue

            vslot_used_sizes.append(vslot.VDiskMetrics.AllocatedSize)
            vslot_fair_size = pdisk_map[pdisk_id].PDiskMetrics.TotalSize / pdisk_slot_usage_map[pdisk_id]
            vslot_fair_sizes.append(vslot_fair_size)

        min_vslot_fair_size = apply_func(min, vslot_fair_sizes)
        max_vslot_used_size = apply_func(max, vslot_used_sizes)
        min_vslot_used_size = apply_func(min, vslot_used_sizes)
        group_size = len(group.VSlotId)

        min_fair_size += min_vslot_fair_size * group_size
        max_used_size += max_vslot_used_size * group_size

        if min_vslot_fair_size > 0:
            vslot_fair_usages.append(min_vslot_used_size / min_vslot_fair_size)
        else:
            vslot_fair_usages.append(0.0)

    estimated_usage = max_used_size / min_fair_size if min_fair_size else 0.0
    max_vslot_fair_usage = apply_func(max, vslot_fair_usages)
    mean_vslot_fair_usage = apply_func(sum, vslot_fair_usages) / len(vslot_fair_usages) if len(vslot_fair_usages) > 0 else 0.0
    std_dev_vslot_fair_usage = math.sqrt(sum((x - mean_vslot_fair_usage)**2 for x in vslot_fair_usages) / len(vslot_fair_usages)) if len(vslot_fair_usages) > 0 else 0.0
    groups_fair_count = math.ceil(len(vslot_fair_usages) * estimated_usage / 0.85)

    res = {}
    res['EstimatedUsage'] = estimated_usage
    res['MaxVDiskEstimatedUsage'] = max_vslot_fair_usage
    res['MeanVDiskEstimatedUsage'] = mean_vslot_fair_usage
    res['StdDevVDiskEstimatedUsage'] = std_dev_vslot_fair_usage
    res['GroupsForEstimatedUsage@85'] = groups_fair_count
    return res


def do(args):
    all_columns = [
        'BoxId:PoolId',
        'PoolName',
        'BoxId',
        'PoolId',
        'ErasureSpecies',
        'Kind',
        'VDiskKind',
        'Groups_TOTAL',
        'Groups_UNKNOWN',
        'Groups_FULL',
        'Groups_PARTIAL',
        'Groups_DEGRADED',
        'Groups_DISINTEGRATED',
        'GroupsForEstimatedUsage@85',
        'VDisks_TOTAL',
        'VDisks_READY',
        'VDisks_ERROR',
        'VDisks_REPLICATING',
        'VDisks_INIT_PENDING',
        'Usage',
        'AvailableSize',
        'UsedSize',
        'TotalSize',
        'EstimatedUsage',
        'MaxVDiskEstimatedUsage',
        'MeanVDiskEstimatedUsage',
        'StdDevVDiskEstimatedUsage',
        'ItemConfigGeneration',
    ]
    visible_columns = [
        'BoxId:PoolId',
        'PoolName',
        'ErasureSpecies',
        'Kind',
        'Groups_TOTAL',
        'VDisks_TOTAL',
    ]
    col_units = {
        'Usage': '%',
        'AvailableSize': 'bytes',
        'UsedSize': 'bytes',
        'TotalSize': 'bytes',
        'EstimatedUsage': '%',
        'MaxVDiskEstimatedUsage': '%',
        'MeanVDiskEstimatedUsage': '%',
        'StdDevVDiskEstimatedUsage': '%',
    }

    if args.show_vdisk_status or args.all_columns:
        visible_columns.extend(['VDisks_READY', 'VDisks_ERROR', 'VDisks_REPLICATING', 'VDisks_INIT_PENDING'])

    if args.show_vdisk_usage or args.all_columns:
        visible_columns.extend(['Usage', 'AvailableSize', 'UsedSize', 'TotalSize'])

    if args.show_vdisk_estimated_usage or args.all_columns:
        visible_columns.extend(['GroupsForEstimatedUsage@85', 'EstimatedUsage', 'MaxVDiskEstimatedUsage', 'MeanVDiskEstimatedUsage', 'StdDevVDiskEstimatedUsage'])

    if args.show_group_status or args.all_columns:
        visible_columns.extend(['Groups_UNKNOWN', 'Groups_FULL', 'Groups_PARTIAL', 'Groups_DEGRADED', 'Groups_DISINTEGRATED'])

    table_output = table.TableOutput(all_columns, col_units=col_units, default_visible_columns=visible_columns)

    base_config_and_storage_pools = common.fetch_base_config_and_storage_pools()
    base_config = base_config_and_storage_pools['BaseConfig']

    box_pool_map = defaultdict(dict)

    group_map = common.build_group_map(base_config)
    for group_id, group in group_map.items():
        pool = box_pool_map[group.BoxId, group.StoragePoolId]

        if 'groups' not in pool:
            pool['groups'] = defaultdict(int)
        groups = pool['groups']

        for key in ['Groups_TOTAL', 'Groups_' + kikimr_bsconfig.TGroupStatus.E.Name(group.OperatingStatus)]:
            groups[key] += 1

        if 'groups_list' not in pool:
            pool['groups_list'] = []
        pool['groups_list'].append(group)

    vslot_map = common.build_vslot_map(base_config)
    for vslot_id, vslot in vslot_map.items():
        if vslot.GroupId not in group_map:
            continue

        group = group_map[vslot.GroupId]
        pool = box_pool_map[group.BoxId, group.StoragePoolId]

        if 'vslots' not in pool:
            pool['vslots'] = defaultdict(int)
        vslots = pool['vslots']

        vslots['UsedSize'] += vslot.VDiskMetrics.AllocatedSize
        vslots['TotalSize'] += vslot.VDiskMetrics.AllocatedSize
        vslots['AvailableSize'] += vslot.VDiskMetrics.AvailableSize
        vslots['TotalSize'] += vslot.VDiskMetrics.AvailableSize

        for key in ['VDisks_TOTAL', 'VDisks_' + vslot.Status]:
            vslots[key] += 1

    rows = []
    for sp in base_config_and_storage_pools['StoragePools']:
        row = {}
        row['BoxId:PoolId'] = '[%u:%u]' % (sp.BoxId, sp.StoragePoolId)
        row['BoxId'] = sp.BoxId
        row['PoolId'] = sp.StoragePoolId
        row['PoolName'] = sp.Name
        row['ErasureSpecies'] = sp.ErasureSpecies
        row['Kind'] = sp.Kind
        row['VDiskKind'] = sp.VDiskKind
        row['ItemConfigGeneration'] = sp.ItemConfigGeneration

        pool = box_pool_map[sp.BoxId, sp.StoragePoolId]

        # fill in groups data
        if 'groups' not in pool:
            pool['groups'] = defaultdict(int)
        groups = pool['groups']
        for key, value in groups.items():
            row[key] = value

        # fill in vslots data
        if 'vslots' not in pool:
            pool['vslots'] = defaultdict(int)
        vslots = pool['vslots']
        for key, value in vslots.items():
            row[key] = value

        # fill in per group stat
        if 'groups_list' not in pool:
            pool['groups_list'] = []

        # fill in usage estimations
        pdisk_map = common.build_pdisk_map(base_config)
        pdisk_slot_usage_map = common.build_pdisk_usage_map(base_config, count_donors=True)
        usage_map = calculate_estimated_usage(pdisk_map, pdisk_slot_usage_map, vslot_map, pool['groups_list'])

        for key, value in usage_map.items():
            row[key] = value

        # set missing columns to 0
        for column in visible_columns:
            if column not in row:
                row[column] = 0

        # fill usage at the end
        row['Usage'] = row['UsedSize'] / row['TotalSize'] if row['TotalSize'] != 0 else 0.0

        rows.append(row)

    table_output.dump(rows, args)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/core/protos/ydb_result_set_old.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from ydb.library.yql.public.types import yql_types_pb2 as ydb_dot_library_dot_yql_dot_public_dot_types_dot_yql__types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(ydb/core/protos/ydb_result_set_old.proto\x12\x06YdbOld\x1a\x1cgoogle/protobuf/struct.proto\x1a,ydb/library/yql/public/types/yql_types.proto\",\n\x08\x44\x61taType\x12 \n\x02id\x18\x01 \x01(\x0e\x32\x14.NYql.NProto.TypeIds\"*\n\x0cOptionalType\x12\x1a\n\x04item\x18\x01 \x01(\x0b\x32\x0c.YdbOld.Type\"&\n\x08ListType\x12\x1a\n\x04item\x18\x01 \x01(\x0b\x32\x0c.YdbOld.Type\"%\n\x07Variant\x12\x1a\n\x04item\x18\x01 \x01(\x0b\x32\x0c.YdbOld.Type\"+\n\tTupleType\x12\x1e\n\x08\x65lements\x18\x01 \x03(\x0b\x32\x0c.YdbOld.Type\"2\n\x06Member\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1a\n\x04type\x18\x02 \x01(\x0b\x32\x0c.YdbOld.Type\"-\n\nStructType\x12\x1f\n\x07members\x18\x01 \x03(\x0b\x32\x0e.YdbOld.Member\"D\n\x08\x44ictType\x12\x19\n\x03key\x18\x01 \x01(\x0b\x32\x0c.YdbOld.Type\x12\x1d\n\x07payload\x18\x02 \x01(\x0b\x32\x0c.YdbOld.Type\"\xe5\x02\n\x04Type\x12%\n\tdata_type\x18\x01 \x01(\x0b\x32\x10.YdbOld.DataTypeH\x00\x12-\n\roptional_type\x18\x02 \x01(\x0b\x32\x14.YdbOld.OptionalTypeH\x00\x12%\n\tlist_type\x18\x03 \x01(\x0b\x32\x10.YdbOld.ListTypeH\x00\x12\'\n\ntuple_type\x18\x04 \x01(\x0b\x32\x11.YdbOld.TupleTypeH\x00\x12)\n\x0bstruct_type\x18\x05 \x01(\x0b\x32\x12.YdbOld.StructTypeH\x00\x12%\n\tdict_type\x18\x06 \x01(\x0b\x32\x10.YdbOld.DictTypeH\x00\x12\'\n\x0cvariant_type\x18\x07 \x01(\x0b\x32\x0f.YdbOld.VariantH\x00\x12/\n\tvoid_type\x18\x08 \x01(\x0e\x32\x1a.google.protobuf.NullValueH\x00\x42\x0b\n\ttype_type\"G\n\tValuePair\x12\x1a\n\x03key\x18\x01 \x01(\x0b\x32\r.YdbOld.Value\x12\x1e\n\x07payload\x18\x02 \x01(\x0b\x32\r.YdbOld.Value\"\xc0\x03\n\x05Value\x12\x14\n\nbool_value\x18\x01 \x01(\x08H\x00\x12\x15\n\x0bint32_value\x18\x02 \x01(\x0fH\x00\x12\x16\n\x0cuint32_value\x18\x03 \x01(\x07H\x00\x12\x15\n\x0bint64_value\x18\x04 \x01(\x10H\x00\x12\x16\n\x0cuint64_value\x18\x05 \x01(\x06H\x00\x12\x15\n\x0b\x66loat_value\x18\x06 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x07 \x01(\x01H\x00\x12\x15\n\x0b\x62ytes_value\x18\x08 \x01(\x0cH\x00\x12\x14\n\ntext_value\x18\t \x01(\tH\x00\x12\x35\n\x0fnull_flag_value\x18\n \x01(\x0e\x32\x1a.google.protobuf.NullValueH\x00\x12%\n\x0cnested_value\x18\x0b \x01(\x0b\x32\r.YdbOld.ValueH\x00\x12\x11\n\x07low_128\x18\x0f \x01(\x06H\x00\x12\x1c\n\x05items\x18\x0c \x03(\x0b\x32\r.YdbOld.Value\x12 \n\x05pairs\x18\r \x03(\x0b\x32\x11.YdbOld.ValuePair\x12\x15\n\rvariant_index\x18\x0e \x01(\r\x12\x10\n\x08high_128\x18\x10 \x01(\x06\x42\r\n\x0bvalue_value\"6\n\nColumnMeta\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1a\n\x04type\x18\x02 \x01(\x0b\x32\x0c.YdbOld.Type\"d\n\tResultSet\x12\'\n\x0b\x63olumn_meta\x18\x01 \x03(\x0b\x32\x12.YdbOld.ColumnMeta\x12\x1b\n\x04rows\x18\x02 \x03(\x0b\x32\r.YdbOld.Value\x12\x11\n\ttruncated\x18\x03 \x01(\x08\x42#\n\x11ru.yandex.ydb.oldB\x0bValueProtos\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.core.protos.ydb_result_set_old_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021ru.yandex.ydb.oldB\013ValueProtos\370\001\001'
  _DATATYPE._serialized_start=128
  _DATATYPE._serialized_end=172
  _OPTIONALTYPE._serialized_start=174
  _OPTIONALTYPE._serialized_end=216
  _LISTTYPE._serialized_start=218
  _LISTTYPE._serialized_end=256
  _VARIANT._serialized_start=258
  _VARIANT._serialized_end=295
  _TUPLETYPE._serialized_start=297
  _TUPLETYPE._serialized_end=340
  _MEMBER._serialized_start=342
  _MEMBER._serialized_end=392
  _STRUCTTYPE._serialized_start=394
  _STRUCTTYPE._serialized_end=439
  _DICTTYPE._serialized_start=441
  _DICTTYPE._serialized_end=509
  _TYPE._serialized_start=512
  _TYPE._serialized_end=869
  _VALUEPAIR._serialized_start=871
  _VALUEPAIR._serialized_end=942
  _VALUE._serialized_start=945
  _VALUE._serialized_end=1393
  _COLUMNMETA._serialized_start=1395
  _COLUMNMETA._serialized_end=1449
  _RESULTSET._serialized_start=1451
  _RESULTSET._serialized_end=1551
# @@protoc_insertion_point(module_scope)
